;******************************************************************
;                                                                 *
;   Filename: K8LH 16F1828 Great Race.asm                         *
;     Author: Mike McLaren, K8LH                                  *
;           : Micro Application Consultants                       *
;       Date: 20-Dec-2012                                         *
;                                                                 *
;  A "moving message display" plus a clone of the "great race"    *
;  for the K8LH 16F1828 8x8 Red/Grn (CCR or CAR type) hardware.   *
;                                                                 *
;      MPLab: 8.84    (tabs=8)                                    *
;      MPAsm: 5.44                                                *
;                                                                 *
;******************************************************************

  #include <P16F1828.INC>
  list st=off       ; turn symbol table off in LST file
  errorlevel -302   ; disable bank warning messages
  radix dec         ; use decimal numbers

  __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _MCLRE_OFF
  __CONFIG _CONFIG2, _BORV_LO & _LVP_OFF & _PLLEN_OFF
;
; _PWRTE_OFF    default
; _CP_OFF       default
; _CPD_OFF      default
; _BOREN_ON     default
; _CLKOUTEN_OFF default
; _IESO_ON      default
; _FCMEN_ON     default
;
; _WRT_OFF      default
; _STVREN_ON    default
;

;--< variables >---------------------------------------------------

        cblock 0x20
vram:9              ; video ram, char vram[9]
bent                ; 
ptrl                ; message table pointer lo
ptrh                ; message table pointer hi
rowsel:2            ; row select bit mask
repeat              ; video repeat (scroll speed)
reptmr              ; repeat timer
dotmask             ; segment mask
speed               ;
level               ; game level, 1..7
swnew               ; buffered/debounced switch state
        endc

#define carpos  vram+8

;==================================================================
;  K8LH DelayCy() subsystem macro generates two instructions      =
;==================================================================

clock   equ     4               ; 4, 8, 12, 16, or 20 MHz clock
usecs   equ     clock/4         ; cycles/usec operand multiplier
        radix   dec

DelayCy macro   cycles          ; 8..1031 cycle range
        movlw   ~((cycles-8)/4)
        call    uDelay-(cycles%4)
        endm

;******************************************************************
;  main init (reset vector)                                       *
;******************************************************************

        org     0x0000
init
        banksel ANSELA          ; bank 3                          |B3
        clrf    ANSELA          ; turn ADC off for digital I/O    |B3
        clrf    ANSELB          ;   "                             |B3
        clrf    ANSELC          ;   "                             |B3
;
;  configure INTOSC for 4-MHz operation
;
        banksel	OSCCON		; bank 1			  |B1
        movlw   b'01101000'     ; 4-MHz (no pll)                  |B1
        movwf   OSCCON          ; setup INTOSC                    |B1
waitfs  btfss   OSCSTAT,HFIOFS  ; stable? yes, skip, else         |B1
        bra     waitfs          ; branch and wait                 |B1
;
;  configure and initialize I/O ports
;
        clrf    TRISA           ; RA3 input, others outputs       |B1
        clrf    TRISB           ; RB7..RB4 outputs                |B1
        clrf    TRISC           ; RC7..RC0 outputs                |B1
        banksel	LATA            ; bank 2                          |B2
        clrf    LATA            ;                                 |B2
        clrf    LATB            ;                                 |B2
        movlw   b'11111111'     ;                                 |B2 (CCRow)
        movwf   LATC            ; blank the display               |B2 (CCRow)
;       clrf    LATC            ; blank the display               |B2 (CARow)
        banksel 0               ; bank 0                          |B0
;
;  use internal weak pull-ups for the RA3 mux switch input when
;  using CAR (Common Anode Row) type red/grn 8x8 display
;
        banksel OPTION_REG      ; bank 1                          |B1 (CARow)
        bcf     OPTION_REG,7    ; enable weak pull-ups            |B1 (CARow)
        banksel WPUA            ; bank 4                          |B4 (CARow)
        bsf     WPUA,WPUA3      ; enable RA3 pull-ups             |B4 (CARow)
        banksel 0               ; bank 0 (program default)        |B0

;******************************************************************
;  message display                                                *
;******************************************************************

        banksel vram            ; bank 0                          |B0
        clrf    vram+0          ; vram[0] = 0                     |B0
        clrf    vram+1          ; vram[1] = 0                     |B0
        clrf    vram+2          ; vram[2] = 0                     |B0
        clrf    vram+3          ; vram[3] = 0                     |B0
        clrf    vram+4          ; vram[4] = 0                     |B0
        clrf    vram+5          ; vram[5] = 0                     |B0
        clrf    vram+6          ; vram[6] = 0                     |B0
        clrf    vram+7          ; vram[7] = 0                     |B0
        movlw   msgtbl/256      ; msgtbl address hi               |B0
        movwf   ptrh            ; msgtbl pointer hi               |B0
        movlw   msgtbl%256      ; msgtbl address lo               |B0
        movwf   ptrl            ; msgtbl pointer lo               |B0
        movlw   b'00000001'     ;                                 |B0
        movwf   dotmask         ;                                 |B0
        clrf    reptmr          ; clear repeat timer              |B0

;******************************************************************
;  main loop - message "MERRY CHRISTMAS"                          *
;******************************************************************

msgloop
        movlw   20              ; use this value to adjust        |B0
        movwf   repeat          ; message scrolling speed         |B0
;
;  refresh display
;
refresh
        movlw   b'00010000'     ; prep for row 1 (RB4 bit on)     |B0
        movwf   rowsel+0        ;   "                             |B0
        clrf    rowsel+1        ;   "                             |B0
;
;  refresh eight rows, one row at a time
;
nextrow
        movf    rowsel+1,W      ; row select mask hi              |B0 (CCRow)
;       comf    rowsel+1,W      ; row select mask hi              |B0 (CARow)
        movwf   PORTA           ; onto RA0, RA1, RA2 & RA4 pins   |B0
        movf    rowsel+0,W      ; row select mask lo              |B0 (CCRow)
;       comf    rowsel+0,W      ; row select mask lo              |B0 (CARow)
        movwf   PORTB           ; onto RB4, RB5, RB6 & RB7 pins   |B0

        call    bender          ; get 'bended' new row data       |B0
;
;  refresh an eight pixel row, one pixel at a time
;
nextpix
        movf    bent,W          ; get 'bended' data               |B0
        andwf   dotmask,W       ; only one pixel at a time        |B0
        xorlw   b'11111111'     ; invert for cathodes             |B0 (CCRow)
        movwf   PORTC           ; display new row for...          |B0
        DelayCy(100*usecs-9)    ; 100 usecs minus 9 cycles        |B0
        rlf     dotmask,W       ;                                 |B0
        rlf     dotmask,F       ; shift pixel mask                |B0
        skpc                    ; last pixel? yes, skip, else     |B0
        goto    nextpix         ; do another pixel                |B0

        movlw   b'11111111'     ; turn off cathode columns to     |B0 (CCRow)
        movwf   PORTC           ; blank the display               |B0 (CCRow)
;       clrf    PORTC           ; blank the display               |B0 (CARow)
        lslf    rowsel+0,F      ; shift row select bit mask       |B0
        rlf     rowsel+1,F      ;   "                             |B0
        btfsc   rowsel+1,3      ; on RA3? no, skip, else          |B0
        lslf    rowsel+1,F      ; shift past RA3 position         |B0
        btfss   rowsel+1,5      ; last row? yes, skip, else       |B0
        goto    nextrow         ; refresh the next row            |B0

        decfsz  repeat,F        ; done? yes, skip, else           |B0
        goto    refresh         ; do another full refresh cycle   |B0

        call    scroll          ; scroll display (postinc ptr)    |B0
;
;  check for an active hi switch press on the RA3 mux input
;
        bsf     PORTB,4         ; turn on <left> switch column    |B0
        btfss   PORTA,3         ; sw press?  yes, skip, else      |B0 (CCRow)
;       btfsc   PORTA,3         ; sw press?  yes, skip, else      |B0 (CARow)
        goto    msgloop         ; branch (continue marquee)       |B0

;******************************************************************
;  great race                                                     *
;******************************************************************

carprep
        clrf    vram+0          ; vram[0] = 0                     |B0
        clrf    vram+1          ; vram[1] = 0                     |B0
        clrf    vram+2          ; vram[2] = 0                     |B0
        clrf    vram+3          ; vram[3] = 0                     |B0
        clrf    vram+4          ; vram[4] = 0                     |B0
        clrf    vram+5          ; vram[5] = 0                     |B0
        clrf    vram+6          ; vram[6] = 0                     |B0
        clrf    vram+7          ; vram[7] = 0                     |B0
        movlw   cartbl/256      ; cartbl address hi               |B0
        movwf   ptrh            ; tblptr pointer hi               |B0
        movlw   cartbl%256      ; cartbl address lo               |B0
        movwf   ptrl            ; tblptr pointer lo               |B0
        movlw   28              ;                                 |B0
        movwf   speed           ;                                 |B0
        movlw   b'00000100'     ;                                 |B0
        movwf   carpos          ; initial car position            |B0
        clrf    level           ; initial game level              |B0
;       movlw   b'00000001'     ;                                 |B0
;       movwf   dotmask         ;                                 |B0
;       clrf    reptmr          ; clear repeat timer              |B0

;******************************************************************
;  main loop - great race                                         *
;******************************************************************

carloop
        movf    speed,W         ; speed                           |B0
        movwf   repeat          ;                                 |B0
;
;  refresh display
;
cardisp
        movlw   b'00010000'     ; prep for row 1 (RB4 bit on)     |B0
        movwf   rowsel+0        ;   "                             |B0
        clrf    rowsel+1        ;   "                             |B0
        movlw   vram+0          ; address vram[0]                 |B0
        movwf   FSR0L           ; setup for indirect access       |B0
        clrf    FSR0H           ;                                 |B0
        call    joystick        ; sample switches & move car      |B0
;
;  refresh nine rows (8 green + 1 red), one row at a time
;
disprow
        movf    rowsel+1,W      ;                                 |B0 (CCRow)
;       comf    rowsel+1,W      ;                                 |B0 (CARow)
        movwf   PORTA           ;                                 |B0
        movf    rowsel+0,W      ;                                 |B0 (CCRow)
;       comf    rowsel+0,W      ;                                 |B0 (CARow)
        movwf   PORTB           ;                                 |B0
;
;  sample one of nine mux'd switches during each row update
;
        iorwf   swnew,F         ; indicate pressed                |B0
        btfss   PORTA,3         ; is it pressed? yes, skip, else  |B0 (CCRow)
;       btfsc   PORTA,3         ; is it pressed? yes, skip, else  |B0 (CARow)
        xorwf   swnew,F         ; indicate released               |B0
;
;  refresh an eight pixel row, one pixel at a time
;
dopixel
        movf    INDF0,W         ; wreg = vram[row]                |B0
        andwf   dotmask,W       ; only one pixel at a time        |B0
        xorlw   b'11111111'     ; invert for cathodes             |B0 (CCRow)
        movwf   PORTC           ; display new row                 |B0
        DelayCy(100*usecs)      ; wait approx ~100 usecs          |B0
        rlf     dotmask,W       ;                                 |B0
        rlf     dotmask,F       ; shift pixel mask                |B0
        skpc                    ; last pixel? yes, skip, else     |B0
        goto    dopixel         ; do another pixel                |B0

        moviw   FSR0++          ; bump FSR0 for next row          |B0
        movlw   b'11111111'     ; turn off cathode columns to     |B0 (CCRow)
        movwf   PORTC           ; blank the display               |B0 (CCRow)
;       clrf    PORTC           ; blank the display               |B0 (CARow)
        lslf    rowsel+0,F      ; shift row select bit mask       |B0
        rlf     rowsel+1,F      ;   "                             |B0
        btfsc   rowsel+1,3      ; on RA3? no, skip, else          |B0
        lslf    rowsel+1,F      ; shift past RA3 position         |B0
        btfss   rowsel+1,6      ; last row? yes, skip, else       |B0
        goto    disprow         ; refresh the next row            |B0
;
;  check for collision
;
        movf    carpos,W        ;                                 |B0
        andwf   vram+7,W        ; collision with wall?            |B0
        skpz                    ; no, skip, else                  |B0
        goto    endprep         ; exit (game over)                |B0

        decfsz  repeat,F        ; done? yes, skip, else           |B0
        goto    cardisp         ; do another full refresh cycle   |B0

        call    scroll          ; scroll display (postinc ptr)    |B0
        btfss   level,6         ; end of game? yes, skip, else    |B0
        goto    carloop         ; branch (continue game)          |B0

;******************************************************************
;  end of game (display collision level or happy face)            *
;******************************************************************

endprep
        movlw   200000/800      ; setup switch delay time for a   |B0
        movwf   reptmr          ; 200 msec minimum display time   |B0
;
;  display collision level or the happy face until <left> press
;
endgame
        movlw   endtbl/256      ; endtbl address hi               |B0
        movwf   ptrh            ; tblptr pointer hi               |B0
        movlw   endtbl%256      ; endtbl address lo               |B0
        addwf   level,W         ; add 'level'                     |B0
        movwf   ptrl            ; tblptr pointer lo               |B0
        skpnc                   ; overflow? no, skip, else        |B0
        incf    ptrh,F          ; bump tblptr hi                  |B0
        movlw   b'00010000'     ; prep for row 1 (RB4 bit on)     |B0
        movwf   rowsel+0        ;   "                             |B0
        clrf    rowsel+1        ;   "                             |B0
;
;  refresh eight green rows, one row at a time
;
rowloop
        movf    rowsel+1,W      ;                                 |B0 (CCRow)
;       comf    rowsel+1,W      ;                                 |B0 (CARow)
        movwf   PORTA           ;                                 |B0
        movf    rowsel+0,W      ;                                 |B0 (CCRow)
;       comf    rowsel+0,W      ;                                 |B0 (CARow)
        movwf   PORTB           ;                                 |B0
        call    scroll+14       ; put new row data in vram[0]     |B0
;
;  refresh an eight pixel row, one pixel at a time
;
nextdot
        movf    vram+0,W        ; get new row data                |B0
        andwf   dotmask,W       ; only one pixel at a time        |B0
        xorlw   b'11111111'     ; invert for cathode columns      |B0 (CCRow)
        movwf   PORTC           ; display new row                 |B0
        DelayCy(100*usecs)      ; wait approx ~100 usecs          |B0
        rlf     dotmask,W       ;                                 |B0
        rlf     dotmask,F       ; shift pixel mask                |B0
        skpc                    ; last pixel? yes, skip, else     |B0
        goto    nextdot         ; do another pixel                |B0

        movlw   b'11111111'     ;                                 |B0 (CCRow)
        movwf   PORTC           ; blank the display               |B0 (CCRow)
;       clrf    PORTC           ; blank the display               |B0 (CARow)
        lslf    rowsel+0,F      ; shift row select bit mask       |B0
        rlf     rowsel+1,F      ;   "                             |B0
        btfsc   rowsel+1,3      ; on RA3? no, skip, else          |B0
        lslf    rowsel+1,F      ; shift past RA3 position         |B0
        btfss   rowsel+1,5      ; last row? yes, skip, else       |B0
        goto    rowloop         ; branch (refresh next row)       |B0

        movf    reptmr,F        ; repeat timer running?           |B0
        skpz                    ; repeat ok? yes, skip, else      |B0
        decf    reptmr,F        ; decrement repeat timer          |B0
        skpz                    ; repeat ok? yes, skip, else      |B0
        goto    endgame         ;                                 |B0
;
;  check for <left> switch press to restart game
;
        bsf     PORTB,4         ; turn on <left> switch column    |B0
        btfss   PORTA,3         ; sw press?  yes, skip, else      |B0 (CCRow)
;       btfsc   PORTA,3         ; sw press?  yes, skip, else      |B0 (CARow)
        goto    endgame         ; branch (continue marquee)       |B0
        goto    carprep         ; branch (display graphic again)  |B0

;******************************************************************
;  joystick
;
joystick
        movf    reptmr,F        ; repeat timer running?           |B0
        skpz                    ; repeat ok? yes, skip, else      |B0
        decf    reptmr,F        ; decrement repeat timer          |B0
        skpz                    ; repeat ok? yes, skip, else      |B0
        clrf    swnew           ; don't sample switches           |B0
joy1
        btfss   swnew,4         ; left press? yes, skip, else     |B0
        bra     joy2            ; branch                          |B0
        lslf    carpos,W        ; wreg = shifted car data         |B0
        btfss   WREG,7          ; edge limit? yes, skip, else     |B0
        movwf   carpos          ; update car position             |B0
        movlw   16              ;                                 |B0
        movwf   reptmr          ; reset repeat interval timer     |B0
joy2
        btfss   swnew,5         ; right press? yes, skip, else    |B0
        return                  ;                                 |B0
        lsrf    carpos,W        ; wreg = shifted car data         |B0
        btfss   WREG,0          ; right edge? no, skip, else      |B0
        movwf   carpos          ; update car position             |B0
        movlw   16              ;                                 |B0
        movwf   reptmr          ; reset repeat interval timer     |B0
        return                  ;                                 |B0

;******************************************************************
;                                                                 *
;******************************************************************

rdtable
        movf    ptrh,W          ; pointer hi                      |B0
        movwf   PCLATH          ; setup PC hi                     |B0
        movf    ptrl,W          ; pointer lo                      |B0
        movwf   PCL             ; write PC lo (jump into table)   |B0

;******************************************************************
;  K8LH DelayCy() subsystem 8 bit timing subroutine               *
;******************************************************************

        nop                     ; entry point for (delay%4) == 3  |B0
        nop                     ; entry point for (delay%4) == 2  |B0
        nop                     ; entry point for (delay%4) == 1  |B0
uDelay  addlw   1               ; entry point for (delay%4) == 0  |B0
        skpz                    ; overflow? yes, skip, else       |B0
        goto    uDelay          ; branch (4 cycle loop)           |B0
        return                  ;                                 |B0

;******************************************************************
;  scroll (shift the display buffer)                              *
;******************************************************************

scroll
        movf    vram+6,W        ;                                 |B0
        movwf   vram+7          ; vram[7] = vram[6]               |B0
        movf    vram+5,W        ;                                 |B0
        movwf   vram+6          ; vram[6] = vram[5]               |B0
        movf    vram+4,W        ;                                 |B0
        movwf   vram+5          ; vram[5] = vram[4]               |B0
        movf    vram+3,W        ;                                 |B0
        movwf   vram+4          ; vram[4] = vram[3]               |B0
        movf    vram+2,W        ;                                 |B0
        movwf   vram+3          ; vram[3] = vram[2]               |B0
        movf    vram+1,W        ;                                 |B0
        movwf   vram+2          ; vram[2] = vram[1]               |B0
        movf    vram+0,W        ;                                 |B0
        movwf   vram+1          ; vram[1] = vram[0]               |B0
        call    rdtable         ; get next table element          |B0
        movwf   vram+0          ; vram[0] = rdtable();            |B0
;
;  post increment table pointer
;
        incf    ptrl,F          ; bump table pointer lo           |B0
        skpnz                   ; overflow? no, skip, else        |B0
        incf    ptrh,F          ; bump table pointer hi           |B0
        return                  ;                                 |B0

;******************************************************************
;  data bender allows scrolling text side-ways on the display     *
;******************************************************************

bender
        rrf     vram+7,W        ;                                 |B0
        rrf     vram+7,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+6,W        ;                                 |B0
        rrf     vram+6,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+5,W        ;                                 |B0
        rrf     vram+5,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+4,W        ;                                 |B0
        rrf     vram+4,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+3,W        ;                                 |B0
        rrf     vram+3,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+2,W        ;                                 |B0
        rrf     vram+2,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+1,W        ;                                 |B0
        rrf     vram+1,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        rrf     vram+0,W        ;                                 |B0
        rrf     vram+0,F        ;                                 |B0
        rlf     bent,F          ;                                 |B0
        return                  ;                                 |B0

;******************************************************************
;  message table                                                  *
;******************************************************************

msgtbl
        retlw   b'10000000'
        retlw   b'10000000'
        retlw   b'10000000'
        retlw   b'10000000'
        retlw   b'10000000'
        retlw   b'10000000'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'M'
        retlw   b'00000010'
        retlw   b'00000100'
        retlw   b'00000010'
        retlw   b'01111111'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'E'
        retlw   b'01001001'
        retlw   b'01001001'
        retlw   b'01001001'
        retlw   b'01000001'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'R'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'01110110'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'R'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'01110110'
        retlw   b'00000000'
        retlw   b'00000111'     ; 'Y'
        retlw   b'00001000'
        retlw   b'01110000'
        retlw   b'00001000'
        retlw   b'00000111'
        retlw   b'00000000'
        retlw   b'00000000'     ; ' '
        retlw   b'00000000'
        retlw   b'00000000'
        retlw   b'00000000'
        retlw   b'00111110'     ; 'C'
        retlw   b'01000001'
        retlw   b'01000001'
        retlw   b'01000001'
        retlw   b'00100010'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'H'
        retlw   b'00001000'
        retlw   b'00001000'
        retlw   b'00001000'
        retlw   b'01111111'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'R'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'01110110'
        retlw   b'00000000'
        retlw   b'01000001'     ; 'I'
        retlw   b'01111111'
        retlw   b'01000001'
        retlw   b'00000000'
        retlw   b'00000110'     ; 'S'
        retlw   b'01001001'
        retlw   b'01001001'
        retlw   b'01001001'
        retlw   b'00110000'
        retlw   b'00000000'
        retlw   b'00000001'     ; 'T'
        retlw   b'00000001'
        retlw   b'01111111'
        retlw   b'00000001'
        retlw   b'00000001'
        retlw   b'00000000'
        retlw   b'01111111'     ; 'M'
        retlw   b'00000010'
        retlw   b'00000100'
        retlw   b'00000010'
        retlw   b'01111111'
        retlw   b'00000000'
        retlw   b'01111110'     ; 'A'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'00001001'
        retlw   b'01111110'
        retlw   b'00000000'
        retlw   b'00000110'     ; 'S'
        retlw   b'01001001'
        retlw   b'01001001'
        retlw   b'01001001'
        retlw   b'00110000'
        retlw   b'00000000'
        retlw   b'00000000'
        retlw   b'00000000'
        retlw   b'00000000'
;
;  last entry wraps around to the beginning of the message table
;
        movlw   msgtbl/256      ; msgtbl address hi               |B0
        movwf   ptrh            ; reset pointer hi                |B0
        movlw   msgtbl%256      ; msgtbl address lo               |B0
        movwf   ptrl            ; reset pointer lo                |B0
        goto    rdtable         ; return msgtbl[0] data           |B0

;******************************************************************
;                                                                 *
;******************************************************************

cartbl
        retlw   b'10000001'     ; 
        retlw   b'10000001'     ; 
        retlw   b'11000001'     ; 
        retlw   b'11000001'
        retlw   b'11100001'
        retlw   b'11100001'
        retlw   b'11110001'
        retlw   b'11110001'
        retlw   b'11111001'
        retlw   b'11110001'
        retlw   b'11100001'
        retlw   b'11000011'
        retlw   b'10000111'
        retlw   b'10001111'
        retlw   b'10000111'
        retlw   b'10000011'
        retlw   b'10110001'
        retlw   b'10111001'
        retlw   b'10111001'
        retlw   b'10111001'
        retlw   b'10010001'
        retlw   b'10000001'
        retlw   b'10000001'
        retlw   b'11000011'
        retlw   b'11100111'
        retlw   b'11000011'
        retlw   b'10000001'
        retlw   b'10011001'
        retlw   b'10011001'
        retlw   b'10111101'
        retlw   b'10111101'
        retlw   b'10011001'
        retlw   b'10011001'
        retlw   b'10000001'
        retlw   b'11000011'
        retlw   b'11100111'
        retlw   b'11000011'
        retlw   b'10000001'
        retlw   b'10000001'
        retlw   b'10010001'
        retlw   b'10110001'
        retlw   b'10111001'
        retlw   b'10111001'
        retlw   b'10011001'
        retlw   b'10000001'
        retlw   b'10000011'
        retlw   b'10000111'
        retlw   b'10001111'
        retlw   b'10000111'
        retlw   b'11000001'
        retlw   b'11100001'
        retlw   b'11110001'
        retlw   b'11111001'
        retlw   b'11111101'
        retlw   b'11111001'
        retlw   b'11110001'
        retlw   b'10000001'
        retlw   b'10000001'
        retlw   b'10110001'
        retlw   b'10110001'
        retlw   b'10110001'
;
;  end of table, increase level and speed and reset table index
;
        movlw   8               ;                                 |B0
        addwf   level,F         ; increase level                  |B0
        decf    speed,F         ; increase speed for next level   |B0
        movlw   cartbl/256      ; cartbl address hi               |B0
        movwf   ptrh            ; tblptr pointer hi               |B0
        movlw   cartbl%256      ; cartbl address lo               |B0
        movwf   ptrl            ; tblptr pointer lo               |B0
        goto    rdtable         ; return cartbl[0] data           |B0

;******************************************************************
;                                                                 *
;******************************************************************

endtbl
        retlw   b'00011000'     ; "1"
        retlw   b'00111000'
        retlw   b'01111000'
        retlw   b'00011000'
        retlw   b'00011000'
        retlw   b'00011000'
        retlw   b'01111110'
        retlw   b'01111110'
        retlw   b'00111000'     ; "2"
        retlw   b'01111100'
        retlw   b'11000110'
        retlw   b'00001100'
        retlw   b'00011000'
        retlw   b'00110000'
        retlw   b'01111110'
        retlw   b'11111110'
        retlw   b'00111100'     ; "3"
        retlw   b'01111110'
        retlw   b'00000110'
        retlw   b'00111110'
        retlw   b'00111110'
        retlw   b'00000110'
        retlw   b'01111110'
        retlw   b'00111100'
        retlw   b'01100110'     ; "4"
        retlw   b'01100110'
        retlw   b'01100110'
        retlw   b'01100110'
        retlw   b'01111110'
        retlw   b'00111110'
        retlw   b'00000110'
        retlw   b'00000110'
        retlw   b'01111110'     ; "5"
        retlw   b'01111110'
        retlw   b'01100000'
        retlw   b'01111100'
        retlw   b'01111110'
        retlw   b'00000110'
        retlw   b'01111110'
        retlw   b'01111100'
        retlw   b'00111110'     ; "6"
        retlw   b'01111110'
        retlw   b'01100000'
        retlw   b'01111100'
        retlw   b'01111110'
        retlw   b'01100110'
        retlw   b'01111110'
        retlw   b'00111100'
        retlw   b'01111111'     ; "7"
        retlw   b'01111111'
        retlw   b'00000011'
        retlw   b'00000110'
        retlw   b'00001100'
        retlw   b'00011000'
        retlw   b'00110000'
        retlw   b'01100000'
        retlw   b'00111110'     ; "8"
        retlw   b'01100011'
        retlw   b'01100011'
        retlw   b'00111110'
        retlw   b'01100011'
        retlw   b'01100011'
        retlw   b'00111110'
        retlw   b'00000000'
        retlw   b'00111100'     ; "smiley face"
        retlw   b'01000010'
        retlw   b'10100101'
        retlw   b'10000001'
        retlw   b'10100101'
        retlw   b'10011001'
        retlw   b'01000010'
        retlw   b'00111100'

;******************************************************************
        end